@@ -18,11 +18,11 @@ Params::Validate::Strict - Validates a set of parameters against a schema
 
 =head1 VERSION
 
-Version 0.20
+Version 0.21
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 =head1 SYNOPSIS
 
@@ -115,7 +115,7 @@ Custom types allow you to define validation rules once and reuse them throughout
 making your validation logic more maintainable and readable.
 
 Each custom type is defined as a hash reference containing the same validation rules available for regular parameters
-(C<type>, C<min>, C<max>, C<matches>, C<memberof>, C<callback>, etc.).
+(C<type>, C<min>, C<max>, C<matches>, C<memberof>, C<notmemberof>, C<callback>, etc.).
 
   my $custom_types = {
     email => {
@@ -196,6 +196,130 @@ The parameter must be an object of type C<isa>.
 
 The parameter must be a member of the given arrayref.
 
+  status => {
+    type => 'string',
+    memberof => ['draft', 'published', 'archived']
+  }
+
+  priority => {
+    type => 'integer',
+    memberof => [1, 2, 3, 4, 5]
+  }
+
+For string types, the comparison is case-sensitive by default. Use the C<case_sensitive>
+flag to control this behavior:
+
+  # Case-sensitive (default) - must be exact match
+  code => {
+    type => 'string',
+    memberof => ['ABC', 'DEF', 'GHI']
+    # 'abc' will fail
+  }
+
+  # Case-insensitive - any case accepted
+  code => {
+    type => 'string',
+    memberof => ['ABC', 'DEF', 'GHI'],
+    case_sensitive => 0
+    # 'abc', 'Abc', 'ABC' all pass, original case preserved
+  }
+
+For numeric types (C<integer>, C<number>, C<float>), the comparison uses numeric
+equality (C<==> operator):
+
+  rating => {
+    type => 'number',
+    memberof => [0.5, 1.0, 1.5, 2.0]
+  }
+
+Note that C<memberof> cannot be combined with C<min> or C<max> constraints as they
+serve conflicting purposes - C<memberof> defines an explicit whitelist while C<min>/C<max>
+define ranges.
+
+=item * C<notmemberof>
+
+The parameter must not be a member of the given arrayref (blacklist).
+This is the inverse of C<memberof>.
+
+  username => {
+    type => 'string',
+    notmemberof => ['admin', 'root', 'system', 'administrator']
+  }
+
+  port => {
+    type => 'integer',
+    notmemberof => [22, 23, 25, 80, 443]  # Reserved ports
+  }
+
+Like C<memberof>, string comparisons are case-sensitive by default but can be controlled
+with the C<case_sensitive> flag:
+
+  # Case-sensitive (default)
+  username => {
+    type => 'string',
+    notmemberof => ['Admin', 'Root']
+    # 'admin' would pass, 'Admin' would fail
+  }
+
+  # Case-insensitive
+  username => {
+    type => 'string',
+    notmemberof => ['Admin', 'Root'],
+    case_sensitive => 0
+    # 'admin', 'ADMIN', 'Admin' all fail
+  }
+
+The blacklist is checked after any C<transform> rules are applied, allowing you to
+normalize input before checking:
+
+  username => {
+    type => 'string',
+    transform => sub { lc($_[0]) },  # Normalize to lowercase
+    notmemberof => ['admin', 'root', 'system']
+  }
+
+C<notmemberof> can be combined with other validation rules:
+
+  username => {
+    type => 'string',
+    notmemberof => ['admin', 'root', 'system'],
+    min => 3,
+    max => 20,
+    matches => qr/^[a-z0-9_]+$/
+  }
+
+=item * C<case_sensitive>
+
+A boolean value indicating whether string comparisons should be case-sensitive.
+This flag affects the C<memberof> and C<notmemberof> validation rules.
+The default value is C<1> (case-sensitive).
+
+When set to C<0>, string comparisons are performed case-insensitively, allowing values
+with different casing to match. The original case of the input value is preserved in
+the validated output.
+
+  # Case-sensitive (default)
+  status => {
+    type => 'string',
+    memberof => ['Draft', 'Published', 'Archived'] # Input 'draft' will fail - must match exact case
+  }
+
+  # Case-insensitive
+  status => {
+    type => 'string',
+    memberof => ['Draft', 'Published', 'Archived'],
+    case_sensitive => 0 # Input 'draft', 'DRAFT', or 'DrAfT' will all pass
+  }
+
+  country_code => {
+    type => 'string',
+    memberof => ['US', 'UK', 'CA', 'FR'],
+    case_sensitive => 0  # Accept 'us', 'US', 'Us', etc.
+  }
+
+This flag has no effect on numeric types (C<integer>, C<number>, C<float>) as numbers
+do not have case.
+
 =item * C<min>
 
 The minimum length (for strings), value (for numbers) or number of keys (for hashrefs).
@@ -413,8 +537,6 @@ handles edge cases appropriately.
 It is the responsibilty of the transformer to ensure that the type of the returned value is correct,
 since that is what will be validated.
 
-=back
-
 Many validators also allow a code ref to be passed so that you can create your own, conditional validation rule, e.g.:
 
   $schema = {
@@ -427,6 +549,129 @@ Many validators also allow a code ref to be passed so that you can create your o
     }
   }
 
+=item * C<cross_validation>
+
+A reference to a hash that defines validation rules that depend on more than one parameter.
+Cross-field validations are performed after all individual parameter validations have passed,
+allowing you to enforce business logic that requires checking relationships between different fields.
+
+Each cross-validation rule is a key-value pair where the key is a descriptive name for the validation
+and the value is a code reference that accepts a hash reference of all validated parameters.
+The subroutine should return C<undef> if the validation passes, or an error message string if it fails.
+
+  my $schema = {
+    password => { type => 'string', min => 8 },
+    password_confirm => { type => 'string' }
+  };
+
+  my $cross_validation = {
+    passwords_match => sub {
+      my $params = shift;
+      return $params->{password} eq $params->{password_confirm}
+        ? undef : "Passwords don't match";
+    }
+  };
+
+  my $validated = validate_strict(
+    schema => $schema,
+    input => $input,
+    cross_validation => $cross_validation
+  );
+
+Common use cases include password confirmation, date range validation, numeric comparisons,
+and conditional requirements:
+
+  # Date range validation
+  my $cross_validation = {
+    date_range_valid => sub {
+      my $params = shift;
+      return $params->{start_date} le $params->{end_date}
+        ? undef : "Start date must be before or equal to end date";
+    }
+  };
+
+  # Price range validation
+  my $cross_validation = {
+    price_range_valid => sub {
+      my $params = shift;
+      return $params->{min_price} <= $params->{max_price}
+        ? undef : "Minimum price must be less than or equal to maximum price";
+    }
+  };
+
+  # Conditional required field
+  my $cross_validation = {
+    address_required_for_delivery => sub {
+      my $params = shift;
+      if ($params->{shipping_method} eq 'delivery' && !$params->{delivery_address}) {
+        return "Delivery address is required when shipping method is 'delivery'";
+      }
+      return undef;
+    }
+  };
+
+Multiple cross-validations can be defined in the same hash, and they are all checked in order.
+If any cross-validation fails, the function will C<croak> with the error message returned by the validation:
+
+  my $cross_validation = {
+    passwords_match => sub {
+      my $params = shift;
+      return $params->{password} eq $params->{password_confirm}
+        ? undef : "Passwords don't match";
+    },
+    emails_match => sub {
+      my $params = shift;
+      return $params->{email} eq $params->{email_confirm}
+        ? undef : "Email addresses don't match";
+    },
+    age_matches_birth_year => sub {
+      my $params = shift;
+      my $current_year = (localtime)[5] + 1900;
+      my $calculated_age = $current_year - $params->{birth_year};
+      return abs($calculated_age - $params->{age}) <= 1
+        ? undef : "Age doesn't match birth year";
+    }
+  };
+
+Cross-validations receive the parameters after individual validation and transformation have been applied,
+so you can rely on the data being in the correct format and type:
+
+  my $schema = {
+    email => {
+      type => 'string',
+      transform => sub { lc($_[0]) }  # Lowercased before cross-validation
+    },
+    email_confirm => {
+      type => 'string',
+      transform => sub { lc($_[0]) }
+    }
+  };
+
+  my $cross_validation = {
+    emails_match => sub {
+      my $params = shift;
+      # Both emails are already lowercased at this point
+      return $params->{email} eq $params->{email_confirm}
+        ? undef : "Email addresses don't match";
+    }
+  };
+
+Cross-validations can access nested structures and optional fields:
+
+  my $cross_validation = {
+    guardian_required_for_minors => sub {
+      my $params = shift;
+      if ($params->{user}{age} < 18 && !$params->{guardian}) {
+        return "Guardian information required for users under 18";
+      }
+      return undef;
+    }
+  };
+
+All cross-validations must pass for the overall validation to succeed.
+
+=back
+
 If a parameter is optional and its value is C<undef>,
 validation will be skipped for that parameter.
 
@@ -885,7 +1130,8 @@ sub validate_strict
 								$ok = 0;
 							}
 						} else {
-							unless(List::Util::any { $_ eq $value } @{$rule_value}) {
+							my $l = lc($value);
+							unless(List::Util::any { (!defined($rules->{'case_sensitive'}) || ($rules->{'case_sensitive'} == 1)) ? $_ eq $value : lc($_) eq $l } @{$rule_value}) {
 								$ok = 0;
 							}
 						}
@@ -904,6 +1150,37 @@ sub validate_strict
 							_error($logger, "validate_strict: Parameter '$key' rule ($rule_value) must be an array reference");
 						}
 					}
+				} elsif($rule_name eq 'notmemberof') {
+					if(!defined($value)) {
+						next;	# Skip if string is undefined
+					}
+					if(ref($rule_value) eq 'ARRAY') {
+						my $ok = 1;
+						if(($rules->{'type'} eq 'integer') || ($rules->{'type'} eq 'number') || ($rules->{'type'} eq 'float')) {
+							if(List::Util::any { $_ == $value } @{$rule_value}) {
+								$ok = 0;
+							}
+						} else {
+							my $l = lc($value);
+							if(List::Util::any { (!defined($rules->{'case_sensitive'}) || ($rules->{'case_sensitive'} == 1)) ? $_ eq $value : lc($_) eq $l } @{$rule_value}) {
+								$ok = 0;
+							}
+						}
+
+						if(!$ok) {
+							if($rules->{'error_message'}) {
+								_error($logger, $rules->{'error_message'});
+							} else {
+								_error($logger, "validate_strict: Parameter '$key' ($value) must not be one of ", join(', ', @{$rule_value}));
+							}
+						}
+					} else {
+						if($rules->{'error_message'}) {
+							_error($logger, $rules->{'error_message'});
+						} else {
+							_error($logger, "validate_strict: Parameter '$key' rule ($rule_value) must be an array reference");
+						}
+					}
 				} elsif ($rule_name eq 'callback') {
 					unless (defined &$rule_value) {
 						_error($logger, "validate_strict: callback for '$key' must be a code reference");
@@ -988,6 +1265,8 @@ sub validate_strict
 					# Handled inline
 				} elsif($rule_name eq 'transform') {
 					# Handled before the loop
+				} elsif($rule_name eq 'case_sensitive') {
+					# Handled inline
 				} elsif($rule_name eq 'schema') {
 					# Nested schema Run the given schema against each element of the array
 					if($rules->{'type'} eq 'arrayref') {
@@ -1060,6 +1339,19 @@ sub validate_strict
 		$validated_args{$key} = $value;
 	}
 
+	if(my $cross_validation = $params->{'cross_validation'}) {
+		foreach my $validator_name(keys %{$cross_validation}) {
+			my $validator = $cross_validation->{$validator_name};
+			if((!ref($validator)) || (ref($validator) ne 'CODE')) {
+				_error($logger, "validate_strict: cross_validation $validator is not a code snippet");
+				next;
+			}
+			if(my $error = &{$validator}(\%validated_args, $validator)) {
+				_error($logger, $error);
+			}
+		}
+	}
+
 	return \%validated_args;
 }
 
@@ -1112,6 +1404,7 @@ Nigel Horne, C<< <njh at nigelhorne.com> >>
         matches: REGEX;
         nomatch: REGEX;
         memberof: seq VALUE;
+        notmemberof: seq VALUE;
         callback: FUNCTION;
         isa: TYPE_NAME;
         can: METHOD_NAME
@@ -1123,9 +1416,15 @@ Nigel Horne, C<< <njh at nigelhorne.com> >>
 
     ValidatedResult == PARAM_NAME ⇸ VALUE
 
-    │ ∀ rule: ComplexRule • rule.min ≤ rule.max
-    │ ∀ schema: Schema; args: Arguments •
-    │   dom(validate_strict(schema, args)) ⊆ dom(schema) ∪ dom(args)
+    ∀ rule: ComplexRule •
+      rule.min ≤ rule.max ∧
+      ¬(rule.memberof ∧ rule.min) ∧
+      ¬(rule.memberof ∧ rule.max) ∧
+      ¬(rule.notmemberof ∧ rule.min) ∧
+      ¬(rule.notmemberof ∧ rule.max)
+
+    ∀ schema: Schema; args: Arguments •
+      dom(validate_strict(schema, args)) ⊆ dom(schema) ∪ dom(args)
 
     validate_strict: Schema × Arguments → ValidatedResult
 
